/*
 *	grailpath.h
 *
 *	GRAphical Interface Layer, path building. The path building
 *	and manipulation classes are used to handle filled regions.
 *
 *	(C) Copyright 1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_GRAILPATH_H_
#define	_OPaC_GRAILPATH_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_RECT_H_
#include <OPaC/rect.h>
#endif

OP_INTERFACE (grailpath.h);

/****************************************************************/

#define	MAX_PATH_SEG		64

/****************************************************************/

class OP_PUBLIC OPGrailPath : public OPObject
{
	struct Point
	{
		OPCoord			x, y;				//	point co-ordinates
	};

	struct Segment
	{
		Point			p1, p2;				//	segment extremities
	};
	
	struct Path
	{
		Path*			next;				//	next path object
		Path*			more;				//	more data for this path
		Count			segments;			//	# of segments in path
		Segment			seg[MAX_PATH_SEG];	//	extremities of the segments
	};
	
private:
	OPRect				bounds;				//	bounding box
	
	OPCoord				ox, oy;				//	origin of current (open) path
	OPCoord				cx, cy;				//	current point
	OPCoord				px, py;				//	last point of last segments
	
	Bool				is_path_open;		//	TRUE => current path not closed
	Bool				is_current_ok;		//	TRUE => current point is OK
	Bool				is_optimised;		//	TRUE => path has been optimised
	Bool				is_segment_in_path;	//	TRUE => at least one segment in (sub)path
	
	Path*				path;				//	list of closed paths
	Path*				current_path;		//	current path (being built)
	Path*				current_path_part;	//	part of current path (being built)
	
protected:
	OPGrailPath ();
	virtual ~ OPGrailPath ();
	
	void ClearPath ();
	void InitPathToZero ();
	void AddPathSegment (OPCoord x, OPCoord y);
	void OptimisePath ();
	
public:
	virtual const OPRect& GetBoundingBox () const;
	virtual Bool GetCurrentPoint (OPCoordOut x, OPCoordOut y) const;
	
	//	Path building operators :
	
	virtual Bool NewPath ();
	virtual Bool ClosePath ();
	
	virtual Bool MoveTo (OPCoord x, OPCoord y);
	virtual Bool LineTo (OPCoord x, OPCoord y);
	
	virtual Bool CurveTo ();
	
	virtual Bool Arc (OPCoord x, OPCoord y, OPCoord r, OPCoord a1, OPCoord a2,
					  Bool counterclockwise = TRUE);
	
	virtual Bool ArcTangent (OPCoord x1, OPCoord y1, OPCoord x2, OPCoord y2);
	
	//	Drawing operations applied to the path :
	
	virtual Bool Stroke (OPGrailPortIn port, OPCoord x, OPCoord y);
	virtual Bool Fill (OPGrailPortIn port, OPCoord x, OPCoord y);
	
	OP_DEFINE_CLASS (OPGrailPath, OPObject);
};

#endif /* _OPaC_GRAILPATH_H_ */
