/*
 *	notmacro.h
 *
 *	This exports the macros which will be used to publish information
 *	about the notification messages an object may send.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_NOTMACRO_H_
#define	_OPaC_NOTMACRO_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/object.h>
#endif

OP_INTERFACE (notmacro.h);

/****************************************************************/

#define	OP_NOTIFY_EVENT(c,n)									\
																\
static OPAtomRef notify_##n;									\
const OPaC_NotifyExport opac_export_##c##_##n (#n, & OP_RUNTIME (c), & notify_##n)

#define	OP_NOTIFY_CHANGE(c,f,n)									\
																\
static OPAtomRef change_##f##_##n;								\
const OPaC_NotifyExport opac_export_##c##_##f##_##n (#f, #n, & OP_RUNTIME (c), & change_##f##_##n)


/****************************************************************/

/*
 *	The notification data export class will record the individual
 *	notification definitions which are then handled by OPRuntime.
 */

class OP_PUBLIC OPaC_NotifyExport
{
	friend class OPRuntime;

private:
	static OPaC_NotifyExport*	head;
	static OPaC_NotifyExport*	tail;
private:
	OPaC_NotifyExport*			next;
	const OPRuntimeRef*			runtime;
	const char*					name;
	const char*					field;
	OPAtomRef*					atom;
public:
	OPaC_NotifyExport (const char* n, const OPRuntimeRef* rt, OPAtomRef* at) :
		next (0), runtime (rt), name (n), field (0), atom (at)
	{ (this->tail ? this->tail->next : this->head) = this; this->tail = this; }
	OPaC_NotifyExport (const char* f, const char* n, const OPRuntimeRef* rt, OPAtomRef* at) :
		next (0), runtime (rt), name (n), field (f), atom (at)
	{ (this->tail ? this->tail->next : this->head) = this; this->tail = this; }
};

/****************************************************************/

#endif /* _OPaC_NOTMACRO_H_ */
