/*
 *	file_io.h
 *
 *	The File I/O functions support low-level access to the files. These should
 *	in no case be used directly by the developer !
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SD_FILE_IO_H_
#define	_OPaC_SD_FILE_IO_H_

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif

/*
 *	OPaC low level file access functions :
 */

typedef unsigned long	OPaC_FileDesc;

extern OPaC_FileDesc OPaC_OpenFileRead (OPStringIn name);
extern OPaC_FileDesc OPaC_OpenFileWrite (OPStringIn name);
extern OPaC_FileDesc OPaC_OpenFileAppend (OPStringIn name);

extern Size OPaC_ReadFile (OPaC_FileDesc file, void* buffer, Size size);
extern Size OPaC_WriteFile (OPaC_FileDesc file, const void* buffer, Size size);

extern Bool OPaC_SeekFileRel (OPaC_FileDesc file, Int32 offset);
extern Bool OPaC_SeekFileAbs (OPaC_FileDesc file, Int32 offset);
extern Size OPaC_TellFilePos (OPaC_FileDesc file);
extern Size OPaC_TellFileSize (OPaC_FileDesc file);

extern Bool OPaC_TruncFile (OPaC_FileDesc file);
extern Bool OPaC_ResizeFile (OPaC_FileDesc file, Size size);
extern Bool OPaC_FlushFile (OPaC_FileDesc file);
extern Bool OPaC_CloseFile (OPaC_FileDesc file);

#endif /* _OPaC_SD_FILE_IO_H_ */

