/*
 *	filedir.h
 *
 *	The File Dir class provides access to directories.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SD_FILEDIR_H_
#define	_OPaC_SD_FILEDIR_H_

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif

/****************************************************************/

/*
 *	File information (kind of "stat") is stored in a structure
 *	which is used for File Directory exploration.
 */

struct OPaC_FileInfo
{
	OPStringRef			name;					//	file name (without path)
	Card64				size;					//	file size (64-bit)
	Int64				creation_time;			//	file creation time
	Int64				last_write_time;		//	file modification time
	Int64				last_access_time;		//	file access time
	
	Bool				is_directory;			//	file is a directory
	Bool				is_link;				//	file is a link
	Bool				is_hidden;				//	file is hidden
	
	Bool				is_read_ok;				//	file can be read
	Bool				is_write_ok;			//	file can be written
};

OP_TYPESTRUCT (OPaC_FileInfo);

/****************************************************************/

/*
 *	OPaC low level directory access class :
 */

class OP_PUBLIC OPaC_FileDir
{
protected:
	Bool				is_open;				//	TRUE => file directory is open
	
protected:
	OPaC_FileDir ();
	virtual ~ OPaC_FileDir ();
public:
	static OPaC_FileDir* New ();
	static void Delete (OPaC_FileDir* dir);
	
	virtual Bool Open (OPStringIn path, OPStringIn pattern) = 0;
	virtual Bool Close () = 0;
	virtual Bool FindNext (OPaC_FileInfoOut info) = 0;
	virtual Bool FindNext (OPaC_FileInfoOut info, OPDictOut attr) = 0;
	
	static Bool Properties (OPStringIn name, OPaC_FileInfoOut info);
};

/****************************************************************/

#endif /* _OPaC_SD_FILEDIR_H_ */

