/*
 *	switch.h
 *
 *	Switch class definition. The switch is being used to handle both switch boxes
 *	and radio buttons, as people call them.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SWITCH_H_
#define	_OPaC_SWITCH_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (switch.h);

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_XTWIDGET_H_
#include <OPaC/xtwidget.h>
#endif

/****************************************************************/

/*
 *	Switch class, used to draw both the check boxes and
 *	the radio buttons.
 */

class OP_PUBLIC OPSwitch : public OPControl, public OPxTitleWidget
{
	
private:	
	static OPAtomRef	name_clear_radio;		//	broadcast value "ClearRadio"

protected:
	OPTextTagRef		title;					//	associated title
	Bool				value;					//	switch value, TRUE => being set
	
	Bool				is_radio_button;		//	TRUE => use radion button icon
	Bool				is_check_box;			//	TRUE => use check-box icon
	
protected:
	OPSwitch ();
	virtual ~ OPSwitch ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void Action (Bool clicked);					/* OVERRIDDEN */
	virtual void ChangeGeometry (OPRectIn);				/* OVERRIDDEN */
	virtual void DrawActiveFrame (OPGrailPortIn, OPAdornerIn, OPCoord); /* OVERRIDDEN */
	
	virtual void AllocTitle (Bool alloc);
	
	virtual OPView* ReturnObjectAsView ();
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void InitAsCheckBox ();
	virtual void InitAsRadioButton ();
	
	virtual Bool SetTitle (OPStringIn title);
	virtual Bool SetValue (Bool on);
//	virtual Bool SetAsCheckBox (Bool yes);
	virtual Bool SetAsRadioButton (Bool yes);
	
	virtual void TurnOn ();
	virtual void TurnOff ();
	virtual void Toggle ();
	
//	virtual Bool IsACheckBox () const;
	virtual Bool IsARadioButton () const;
//	virtual Bool IsACheckBox (BoolOut yes) const;
	virtual Bool IsARadioButton (BoolOut yes) const;
	virtual Bool GetTitle (OPStringOut title) const;
	virtual Bool ReturnValue () const;
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	
	virtual void Broadcast (OPAtomIn, OPObjectIn);			/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */

	OP_DEFINE_CLASS (OPSwitch, OPControl);
};

/****************************************************************/

#endif /* _OPaC_SWITCH_H_ */

