/*
 *	tabview.h
 *
 *	The Tabbed View holds many views of which only one can be
 *	visible at a given time.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_TABVIEW_H_
#define	_OPaC_TABVIEW_H_

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

OP_INTERFACE (tabview.h);

/****************************************************************/

/*
 *	Tab view class : this implements a pile of cardboards,
 *	stacked to form a "tabbed view".
 */

class OP_PUBLIC OPTabView : public OPView
{
	
protected:
	OPTabListRef		tabs;					//	tabs
	OPCoord				height;					//	height of tabs
	Count				active;					//	currently active tab
	Count				num;					//	current number of tabs
	OPListRef			views;					//	collection of views

	Bool				need_update;			//	TRUE => update needed
	
protected:
	OPTabView ();
	virtual ~ OPTabView ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn box);			/* OVERRIDDEN */
	virtual void SetTabsParent (Count num, OPViewIn view);
	
	virtual void MakeViewActive (Count num);
	virtual void MakeViewInactive (Count num);
	
	virtual void ActivatedTab (Count num);

public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void UpdateTabs ();
	virtual void OnTabChange (OPObjectIn object, Card32 id);
	
	virtual Bool SetTabNum (Count num);
	virtual Bool SetActiveTab (Count active);
	virtual Bool SetHeight (OPCoord height);
	
	virtual Count GetTabNum () const;
	virtual Count GetActiveTab () const;
	virtual OPCoord GetHeight () const;
	
	virtual Bool GetTabs (OPTabListOut tabs) const;
	virtual Bool GetViews (OPListOut views) const;
	
	//	Inherited methods :
	
	virtual Bool SetWindow (OPWindowIn window);			/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPTabView, OPView);
};

/****************************************************************/

#endif /* _OPaC_TABVIEW_H_ */

