/*
 *	sd_grailfnt.cxx
 *
 *	CAUTION: This file provides an interface for Win32 only.
 *
 *	GRAphical Interface Layer Font representation. This is machine
 *	dependent !
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"

#include <OPaC/database.h>
#include <OPaC/grail.h>
#include <OPaC/new.h>

/****************************************************************/

OP_IMPLEMENT_CLASS (OPGrailFont);

/****************************************************************/

OPGrailFont::OPGrailFont ()
{
	OP_METHOD (OPGrailFont, 0);
	
	this->info  = 0;
	this->name  = OPStringNull;
	this->style = OPStringNull;
	this->size  = 0;
}



OPGrailFont::~OPGrailFont ()
{
	OP_METHOD (~OPGrailFont, 0);
	
	if (this->info && this->info->hfont) {
		DeleteObject (this->info->hfont);
	}
	
	if (this->info) {
		OP_FREE_FROM_ZONE (this->ReturnMemZone (), this->info);
	}
}



void
OPGrailFont::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	super::StoreInstance (storage);
	
	storage << this->name;
	storage << this->style;
	storage << this->size;
	
	storage << OPStorageSync::mark;
}



void
OPGrailFont::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	super::RestoreInstance (storage);
	
	storage >> this->name;
	storage >> this->style;
	storage >> this->size;
	
	storage >> OPStorageSync::mark;
	
	this->RealizeFont ();
}



Bool
OPGrailFont::Initialise (OPStringIn name, OPStringIn style, OPCoord size)
{
	OP_METHOD (Initialise, 3);
	OP_ARG (1, OPStringIn, name);
	OP_ARG (2, OPStringIn, style);
	OP_ARG (3, OPCoord, size);
	
	if (this->info) {
		return FALSE;
	}
	
	if (this->name.IsValid () == FALSE) {
		this->name = OP_NEW_FROM_ZONE (this->ReturnMemZone (), OPString);
	}
	
	if (this->style.IsValid () == FALSE) {
		this->style = OP_NEW_FROM_ZONE (this->ReturnMemZone (), OPString);
	}
	
	this->name->SetString (name);
	this->style->SetString (style);
	
	this->size = size;
	
	return this->RealizeFont ();
}



Bool
OPGrailFont::RealizeFont ()
{
	OP_METHOD (RealizeFont, 0);
	
	if ( (this->name.IsValid ())
	  && (this->style.IsValid ())
	  && (this->size) ) {
		
		if (this->info == 0) {
			this->info = OP_ALLOC_FROM_ZONE (this->ReturnMemZone (), OPaC_GrFont);
			this->info->hfont = 0;
		}
		
		if (this->info->hfont) {
			DeleteObject (this->info->hfont);
			this->info->hfont = 0; 
		}
		
		this->info->logfont.lfHeight        = - (int)(this->size);
		this->info->logfont.lfWidth         = 0;
		this->info->logfont.lfEscapement    = 0;
		this->info->logfont.lfOrientation   = 0;
		this->info->logfont.lfWeight        = 0;			//	should be initialised in another way..
		this->info->logfont.lfItalic        = 0;			//	italic ?
		this->info->logfont.lfUnderline     = 0;
		this->info->logfont.lfStrikeOut     = 0;
		this->info->logfont.lfCharSet       = ANSI_CHARSET;
		this->info->logfont.lfOutPrecision  = OUT_TT_ONLY_PRECIS;
		this->info->logfont.lfClipPrecision = 0;
		this->info->logfont.lfQuality       = PROOF_QUALITY;
		this->info->logfont.lfPitchAndFamily= 0;
		
		//	Extract the string and put it into the logical font structure. If
		//	the string happens to be too long, just truncate it and try anyway.
		
		Count      max  = LF_FACESIZE - 1;
		Count      len  = this->name->CountCharacters ();
		LatinChar* name = this->info->logfont.lfFaceName;
		
		if (len > max) len = max;
		
		this->name->GetStringCopy (name, LF_FACESIZE);
		this->info->hfont = CreateFontIndirect (& this->info->logfont);
		
		HDC     hdc   = GetDC (NULL);							//	don't forget to release later on
		HGDIOBJ ofont = SelectObject (hdc, this->info->hfont);	//	select new font temporarily
		GetTextMetrics (hdc, & this->info->metrics);
		SelectObject (hdc, ofont);
		ReleaseDC (NULL, hdc);
		
		return TRUE;
	}
	
	return FALSE;
}



void
OPGrailFont::GetFontName (OPStringOut name) const
{
	OP_METHOD (GetFontName, 1);
	OP_ARG (1, OPStringOut, name);
	
	name = this->name;
}



void
OPGrailFont::GetFontStyle (OPStringOut name) const
{
	OP_METHOD (GetFontStyle, 1);
	OP_ARG (1, OPStringOut, name);
	
	name = this->style;
}



OPCoord
OPGrailFont::ReturnFontSize () const
{
	OP_METHOD (ReturnFontSize, 0);
	return (unsigned int)(this->size);
}



OPRelCoord
OPGrailFont::ReturnAscender () const
{
	OP_METHOD (ReturnAscender, 0);
	
	return (this->info && this->info->hfont)
		   ? this->info->metrics.tmAscent
		   : 0;
}



OPRelCoord
OPGrailFont::ReturnDescender () const
{
	OP_METHOD (ReturnDescender, 0);
	
	return (this->info && this->info->hfont)
		   ? - this->info->metrics.tmDescent
		   : 0;
}



OPCoord
OPGrailFont::ReturnWidth (const LatinChar* text) const
{
	OP_METHOD (ReturnWidth, 1);
	OP_ARG (1, const LatinChar*, text);
	
	if ( (text == 0)
	  || (text[0] == 0) ) {
		return 0;
	}
	
	OPCoord width = 0;
	
	if ( (this->info != 0)
	  && (this->info->hfont != 0) ) {
		
		HDC     hdc   = GetDC (NULL);
		HGDIOBJ ofont = SelectObject (hdc, this->info->hfont);
		int     length;
		
		for (length = 0; text[length]; length++) {
			
			//	Compute length of given string... All done in the for statement.
		}
		
		OP_ASSERT (ofont);
		
		SIZE size;

		GetTextExtentPoint32 (hdc, text, length, &size);
		width = (OPCoord)(size.cx);
		
		SelectObject (hdc, ofont);
		ReleaseDC (NULL, hdc);
	}
	
	return width;
}



OPCoord
OPGrailFont::ReturnWidth (OPStringIn text) const
{
	OP_METHOD (ReturnWidth, 1);
	OP_ARG (1, OPStringIn, text);
	
	if (text.IsValid () == FALSE) {
		return 0;
	}
	
	Count len = text->CountCharacters ();
	
	return this->ReturnWidth (text, 0, len);
}



OPCoord
OPGrailFont::ReturnWidth (OPStringIn text, Count start, Count length) const
{
	OP_METHOD (ReturnWidth, 3);
	OP_ARG (1, OPStringIn, text);
	OP_ARG (2, Count, start);
	OP_ARG (3, Count, length);
	
	if (text.IsValid () == FALSE) {
		return FALSE;
	}
	
	OPCoord width = 0;
	Count   len   = text->CountCharacters ();
	Count   pos   = start + length;
	
	if ( (this->info != 0)
	  && (this->info->hfont != 0)
	  && (length != 0)
	  && (start < len) ) {
		
		if (pos > len) length -= (pos - len);
		
		LatinChar buffer[64];
		SIZE      size;
		HDC       hdc   = GetDC (NULL);
		HGDIOBJ   ofont = SelectObject (hdc, this->info->hfont);
		
		OP_ASSERT (ofont);
		
		//	Compute the font metrics by text chunks. It does not matter whether
		//	we do it in one call or in many pieces (the latter will be somewhat
		//	slower), since Win16 does not use ligatures !!!
		
		while (length > 64) {
			
			text->ExtractStringCopy (buffer, start, 64);
			
			GetTextExtentPoint32 (hdc, buffer, 64, &size);
			
			width  += (OPCoord)(size.cx);
			start  += 64;
			length -= 64;
		}
		
		text->ExtractStringCopy (buffer, start, length);
		
		GetTextExtentPoint32 (hdc, buffer, (int)(length), &size);
		
		width += (OPCoord)(size.cx);
		
		SelectObject (hdc, ofont);
		ReleaseDC (NULL, hdc);
	}
	
	return width;
}


