/*
 *	textcaption.cxx
 *
 *	The text caption displays a text in a colored frame.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/textcaption.h>
#include <OPaC/grail.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>
#include <OPaC/targetmacro.h>
#include <OPaC/pubmacro.h>

/****************************************************************/

OP_IMPLEMENTATION (textcaption.h);
OP_IMPLEMENT_CLASS (OPTextCaption);

/****************************************************************/

/*
 *	Create an empty text tag. No text and no font are
 *	currently defined.
 */

OPTextCaption::OPTextCaption ()
{
	this->draw_frame_high = FALSE;
	this->draw_frame_low  = TRUE;
	
	this->no_draw_back   = TRUE;
	this->back_color     = OPColorMan::ReturnSystemColorMan ()->FindColor (0.5);
	this->text_color     = OPColorMan::ReturnSystemColorMan ()->FindColor (1.0);
	this->use_auto_color = FALSE;
}

OPTextCaption::~OPTextCaption ()
{
}

/*
 *	Store the text tag : string, font and mode plus the
 *	view's definition.
 */

void
OPTextCaption::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
}



/*
 *	Restore the text tag : string, font and mode plus the
 *	view's definition.
 */

void
OPTextCaption::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
}



Bool
OPTextCaption::SetBackColor (OPColorPixel pixel)
{
	OP_METHOD (SetBackColor, 1);
	OP_ARG (1, OPColorPixel, pixel);
	
	this->back_color = pixel;
	
	return TRUE;
}

OPColorPixel
OPTextCaption::ReturnBackColor () const
{
	OP_METHOD (ReturnBackColor, 0);
	return this->back_color;
}


void
OPTextCaption::Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what)
{
	OP_METHOD (Draw, 3);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	OP_ARG (3, OPDrawOp, what);
	
	if (what & OP_DRAW_BACKGROUND) {
		super::Draw (port, rect, OP_DRAW_BACKGROUND);
		what &= ~OP_DRAW_BACKGROUND;
	}
	
	if (what & OP_DRAW_OBJECT) {
		
		const OPAdornerRef& adorner = port->ReturnAdorner ();
		OPCoord margin = OPLookMan::Current ()->Return3DEffectWidth ();
		
		if (this->draw_frame_high) {
			adorner->DrawRectHigh (this->box);
		}
		
		if (this->draw_frame_low) {
			adorner->DrawRectLow (this->box);
		}
		
		port->SetForeColor (this->back_color);
		port->FillRect (this->box, margin);
	}
	
	if (what) {
		super::Draw (port, rect, what);
	}
}
