/*
 *	bitseteditor.cxx
 *
 *	Fill in a window with the fields needed to edit a bitset (text
 *	tags and associated buttons).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "bitsetspy.h"

#include <OPaC/boxview.h>
#include <OPaC/new.h>
#include <OPaC/pubmacro.h>
#include <OPaC/switch.h>
#include <OPaC/window.h>

extern Bool CreateBitsetEditors (OPObjectIn edit, Count i, OPWindowIn window, OPCoord y, OPCoordOut dx, OPCoordOut dy);


Bool
CreateBitsetEditors (OPObjectIn edit, Count i, OPWindowIn window,
					 OPCoord y, OPCoordOut dx, OPCoordOut dy)
{
	const   MAX_BITSET   = 8;
	Card32  count_bitset = 0;
	OPCoord ox           = 110;
	
	dx = 0;
	dy = 0;
	
	const OPaC_BitsetDataDef* bitset_def[MAX_BITSET] = { 0 };
	Card32 bitset_num[MAX_BITSET] = { 0 };
	
	while (edit->GetPublicDataEx (i, count_bitset,
								  bitset_def[count_bitset],
								  bitset_num[count_bitset])) {
		count_bitset++;
	}
	
	if (count_bitset == 0) {
		return FALSE;
	}
	
	for (Count j = 0; j < count_bitset; j++) {
		
		const OPaC_BitsetDataDef* def = bitset_def[j];
		Card32  num = bitset_num[j];
		OPCoord oy  = y;
		OPCoord hx  = 0;
		OPCoord hy  = 0;
		
		if ( (num) && (def->type == OP_BITSET_DATA_MASK) ) {
			
			OPCoord mx  = 0;
			OPCoord my  = 0;
			
			OPBoxViewRef   box = OP_NEW (OPBoxView);
			OPBitsetSpyRef spy = OP_NEW (OPBitsetSpy);
			
			spy->SetRefObject (edit, i, def->value);
			
			box->SetParent (window);
			box->SetOrigin (ox, oy);
			box->SetTitle (def->name);
			box->SetSize (60, 20);
			
			num--;
			def++;
			
			while ( (num)
				 && ( (def->type == OP_BITSET_DATA_VALUE)
				   || (def->type == OP_BITSET_DATA_BIT) ) ) {
				
				OPSwitchRef flag = OP_NEW (OPSwitch);
				
				flag->SetParent (box);
				flag->SetOrigin (ox+10, oy+16);
				if (def->type == OP_BITSET_DATA_VALUE) {
					flag->InitAsRadioButton ();
					spy->AddSwitch (flag, def->value);
				} else {
					flag->InitAsCheckBox ();
					spy->AddSwitchBit (flag, def->value);
				}
				flag->SetTitle (def->name);
				flag->GetMinSize (mx, my);
				flag->SetSize (mx, my);
				
				oy += my;
				hy += my;
				if (mx > hx) hx = mx;
				num--;
				def++;
			}
			
			hx += 20;
			hy += 24;
			
			box->SetSize (hx, hy);
			
			hx += 10;
			hy += 10;
			
			if (hy > dy) dy = hy;
			
			dx += hx;
			ox += hx;
		}
	}

	return TRUE;
}

