/*
 *	bitsetspy.cxx
 *
 *	Spy changes in the bitset edition window and modify and display
 *	the real objects when needed.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "bitsetspy.h"

#include <OPaC/pubmacro.h>
#include <OPaC/switch.h>
#include <OPaC/targetmacro.h>


APP_IMPLEMENT_CLASS (OPBitsetSpy);

OP_TARGET_OBSERVER	(OPBitsetSpy, OnClick);

/****************************************************************/

OPBitsetSpy::OPBitsetSpy ()
{
	this->value  = (Card32)(0);
	this->mask   = 0;
	this->bitset = 0;
	
	this->ref_mode  = REF_NONE;
	this->ref_index = 0;
}

OPBitsetSpy::~OPBitsetSpy ()
{
}

void
OPBitsetSpy::SetRefObject (OPObjectIn object, Count index, Card32 mask)
{
	OP_METHOD (SetRefObject, 3);
	OP_ARG (1, OPObjectIn, object);
	OP_ARG (2, Count, index);
	OP_ARG (3, Card32, mask);
	
	this->object = object;
	this->ref_mode   = REF_BY_INDEX;
	this->ref_index  = index;
	this->ref_name   = OPAtomNull;
	this->ref_path   = OPStringNull;
	this->mask = mask;
	
	this->object->QueryData (this->ref_index, this->value);
	this->value = (Card32)(this->value) & this->mask;
}

void
OPBitsetSpy::SetRefObject (OPObjectIn object, OPAtomIn name, Card32 mask)
{
	OP_METHOD (SetRefObject, 3);
	OP_ARG (1, OPObjectIn, object);
	OP_ARG (2, OPAtomIn, name);
	OP_ARG (3, Card32, mask);
	
	this->object = object;
	this->ref_mode   = REF_BY_NAME;
	this->ref_index  = 0;
	this->ref_name   = name;
	this->ref_path   = OPStringNull;
	this->mask = mask;
	
	this->object->QueryData (this->ref_name, this->value);
	this->value = (Card32)(this->value) & this->mask;
}

void
OPBitsetSpy::SetRefObjectByPath (OPObjectIn object, OPStringIn path, Card32 mask)
{
	OP_METHOD (SetRefObject, 3);
	OP_ARG (1, OPObjectIn, object);
	OP_ARG (2, OPStringIn, path);
	OP_ARG (3, Card32, mask);
	
	this->object = object;
	this->ref_mode   = REF_BY_NAME;
	this->ref_index  = 0;
	this->ref_name   = OPAtomNull;
	this->ref_path   = path;
	this->mask = mask;
	
	this->object->QueryData (this->ref_name, this->value);
	this->value = (Card32)(this->value) & this->mask;
}

void
OPBitsetSpy::AddSwitch (OPSwitchIn button, Card32 value)
{
	button->AttachObserver (this, "OnClick", value);
	button->SetValue (value == this->value);
}

void
OPBitsetSpy::AddSwitchBit (OPSwitchIn button, Card32 bit)
{
	button->AttachObserver (this, "OnClick", bit);
	button->SetValue (((bit) & ((Card32)(this->value))) ? TRUE : FALSE);
	this->bitset |= bit;
}


void
OPBitsetSpy::OnClick (OPObjectIn src, Card32 id)
{
	if (id & this->bitset) {
		switch (this->ref_mode) {
			case REF_BY_INDEX:
				this->object->QueryData (this->ref_index, this->value);
				this->value = (Card32)(this->value) ^ id;
				this->object->DefineData (this->ref_index, this->value);
				break;
			case REF_BY_NAME:
				this->object->QueryData (this->ref_name, this->value);
				this->value = (Card32)(this->value) ^ id;
				this->object->DefineData (this->ref_name, this->value);
				break;
			case REF_BY_PATH:
				OPObject::QueryData (this->object, this->ref_path, this->value);
				this->value = (Card32)(this->value) ^ id;
				OPObject::DefineData (this->object, this->ref_path, this->value);
				break;
		}
	} else {
		switch (this->ref_mode) {
			case REF_BY_INDEX:
				this->object->QueryData (this->ref_index, this->value);
				this->value = ((Card32)(this->value) & ~this->mask) | id;
				this->object->DefineData (this->ref_index, this->value);
				break;
			case REF_BY_NAME:
				this->object->QueryData (this->ref_name, this->value);
				this->value = ((Card32)(this->value) & ~this->mask) | id;
				this->object->DefineData (this->ref_name, this->value);
				break;
			case REF_BY_PATH:
				OPObject::QueryData (this->object, this->ref_path, this->value);
				this->value = ((Card32)(this->value) & ~this->mask) | id;
				OPObject::DefineData (this->object, this->ref_path, this->value);
				break;
		}
	}
	
	this->object->Action ("Display");
	this->value = (Card32)(this->value) & this->mask;
}
