/*
 *	builder.h
 *
 *	Manage the palette window, the main part of the OPaC dynamic
 *	interface builder.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_BUILDER_H_
#define _OPaC_BUILDER_H_

#include <OPaC/app.h>

APP_EXPORT_CLASS (OPBuilder);

APP_DEFINE_REF (OPBuilder, OPObject);

class OPBuilder : public OPObject
{
	OPWindowRef			ib_window;
	OPWindowRef			w_dialog_no_edit;
	
	OPTabViewRef		tab_view;
	OPTabListRef		tab_list;
	OPViewRef			palette;
	OPViewRef			view_palette;
	OPDragPaletteRef	view_drag;
	OPDBScrollListRef	view_list;
	OPViewRef			icon_palette;
	
	OPObjectRef			edit;
	OPSelViewRef		sel_view;
	OPWindowRef			active_window;
	
	OPWindowRef			o_pane_window;
	OPObjectPaneRef		o_pane_pane;
	OPObjectProviderRef	o_pane_provider;
	OPButtonRef			o_pane_palette_button;
	OPButtonRef			o_pane_lock_button;
	OPButtonRef			o_pane_window_button;
	OPIconRef			o_pane_palette_icon;
	OPIconRef			o_pane_lock_icon;

protected:
	OPBuilder ();
	virtual ~ OPBuilder ();
	
	virtual void SetupPalette ();
	virtual void SetupViewPalette ();
	virtual void SetupIconPalette ();
	virtual void SetupDialogNoEdit ();
	
public:
	virtual void Initialise ();
	virtual void SaveWindow ();
	virtual void CloneView ();
	
	virtual void OnClickPalette ();
	virtual void OnClickLock ();
	
	virtual void SubstituteTarget (OPObjectIn subst);
	virtual void EditSimpleObject (OPObjectIn initiator, OPObjectIn object, OPAtomIn field);
	virtual void SetupObjectBrowser (OPDataSlotIn slot);
	
	virtual void ShowDialogNoEdit ();
	virtual void RemoveDialog (OPObjectIn, Card32 id);
	
	virtual Bool SetEditObject (OPObjectIn object);
	virtual Bool SetSelView (OPSelViewIn sel_view);
	
	virtual Bool GetEditObject (OPObjectOut object) const;
	virtual Bool GetSelView (OPSelViewOut sel_view) const;
	
	virtual void NotifySelectionBegin (OPSelViewIn sel);
	virtual void NotifySelectionEnd (OPSelViewIn sel);
	virtual void NotifyMenuBegin ();
	virtual void NotifyMenuEnd (OPObjectIn, Card32 id);
	
	virtual void NotifyInspectSelection ();
	virtual void NotifyDeleteSelection ();
	
	virtual void NotifyConnectionBegin ();
	virtual void NotifyActiveWindowChanged ();
	
	APP_DEFINE_CLASS (OPBuilder, OPObject);
};

#endif
